package javax.swing;

import java.awt.AWTError;
import java.awt.Insets;
import java.awt.Dimension;
import java.awt.Component;
import java.beans.ConstructorProperties;
import java.awt.Container;
import java.io.Serializable;
import java.awt.LayoutManager2;

public class OverlayLayout implements LayoutManager2, Serializable
{
    private Container target;
    private SizeRequirements[] xChildren;
    private SizeRequirements[] yChildren;
    private SizeRequirements xTotal;
    private SizeRequirements yTotal;
    
    @ConstructorProperties({ "target" })
    public OverlayLayout(Container target) {
        this.target = target;
    }
    
    public final Container getTarget() {
        return this.target;
    }
    
    @Override
    public void invalidateLayout(Container container) {
        this.checkContainer(container);
        this.xChildren = null;
        this.yChildren = null;
        this.xTotal = null;
        this.yTotal = null;
    }
    
    @Override
    public void addLayoutComponent(String s, Component component) {
        this.invalidateLayout(component.getParent());
    }
    
    @Override
    public void removeLayoutComponent(Component component) {
        this.invalidateLayout(component.getParent());
    }
    
    @Override
    public void addLayoutComponent(Component component, Object o) {
        this.invalidateLayout(component.getParent());
    }
    
    @Override
    public Dimension preferredLayoutSize(Container container) {
        this.checkContainer(container);
        this.checkRequests();
        Dimension dimension = new Dimension(this.xTotal.preferred, this.yTotal.preferred);
        Insets insets = container.getInsets();
        dimension.width += insets.left + insets.right;
        dimension.height += insets.top + insets.bottom;
        return dimension;
    }
    
    @Override
    public Dimension minimumLayoutSize(Container container) {
        this.checkContainer(container);
        this.checkRequests();
        Dimension dimension = new Dimension(this.xTotal.minimum, this.yTotal.minimum);
        Insets insets = container.getInsets();
        dimension.width += insets.left + insets.right;
        dimension.height += insets.top + insets.bottom;
        return dimension;
    }
    
    @Override
    public Dimension maximumLayoutSize(Container container) {
        this.checkContainer(container);
        this.checkRequests();
        Dimension dimension = new Dimension(this.xTotal.maximum, this.yTotal.maximum);
        Insets insets = container.getInsets();
        dimension.width += insets.left + insets.right;
        dimension.height += insets.top + insets.bottom;
        return dimension;
    }
    
    @Override
    public float getLayoutAlignmentX(Container container) {
        this.checkContainer(container);
        this.checkRequests();
        return this.xTotal.alignment;
    }
    
    @Override
    public float getLayoutAlignmentY(Container container) {
        this.checkContainer(container);
        this.checkRequests();
        return this.yTotal.alignment;
    }
    
    @Override
    public void layoutContainer(Container container) {
        this.checkContainer(container);
        this.checkRequests();
        int componentCount = container.getComponentCount();
        int[] array = new int[componentCount];
        int[] array2 = new int[componentCount];
        int[] array3 = new int[componentCount];
        int[] array4 = new int[componentCount];
        Dimension size = container.getSize();
        Insets insets = container.getInsets();
        size.width -= insets.left + insets.right;
        size.height -= insets.top + insets.bottom;
        SizeRequirements.calculateAlignedPositions(size.width, this.xTotal, this.xChildren, array, array2);
        SizeRequirements.calculateAlignedPositions(size.height, this.yTotal, this.yChildren, array3, array4);
        for (int i = 0; i < componentCount; ++i) {
            container.getComponent(i).setBounds(insets.left + array[i], insets.top + array3[i], array2[i], array4[i]);
        }
    }
    
    void checkContainer(Container container) {
        if (this.target != container) {
            throw new AWTError("OverlayLayout can't be shared");
        }
        return;
    }
    
    void checkRequests() {
        if (this.xChildren == null || this.yChildren == null) {
            int componentCount = this.target.getComponentCount();
            this.xChildren = new SizeRequirements[componentCount];
            this.yChildren = new SizeRequirements[componentCount];
            for (int i = 0; i < componentCount; ++i) {
                Component component = this.target.getComponent(i);
                Dimension minimumSize = component.getMinimumSize();
                Dimension preferredSize = component.getPreferredSize();
                Dimension maximumSize = component.getMaximumSize();
                this.xChildren[i] = new SizeRequirements(minimumSize.width, preferredSize.width, maximumSize.width, component.getAlignmentX());
                this.yChildren[i] = new SizeRequirements(minimumSize.height, preferredSize.height, maximumSize.height, component.getAlignmentY());
            }
            this.xTotal = SizeRequirements.getAlignedSizeRequirements(this.xChildren);
            this.yTotal = SizeRequirements.getAlignedSizeRequirements(this.yChildren);
        }
    }
}
